@@ -22,11 +22,11 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include <stdio.h>
-#include <unistd.h>
+/* This file (auth.c) handles authentication requests, passing it to the
+ * particular type (auth-passwd, auth-pubkey). */
 
-#include "options.h"
-#include "util.h"
+#include "includes.h"
+#include "dbutil.h"
 #include "session.h"
 #include "buffer.h"
 #include "ssh.h"
@@ -34,12 +34,12 @@
 #include "auth.h"
 #include "authpasswd.h"
 #include "authpubkey.h"
-#include "pwd.h"
 
 static void authclear();
 static int checkusername(unsigned char *username, unsigned int userlen);
 static void send_msg_userauth_banner();
 
+/* initialise the first time for a session, resetting all parameters */
 void authinitialise() {
 
 	ses.authstate.failcount = 0;
@@ -47,11 +47,15 @@ void authinitialise() {
 	
 }
 
+/* Reset the auth state, but don't reset the failcount. This is for if the
+ * user decides to try with a different username etc, and is also invoked
+ * on initialisation */
 static void authclear() {
 	
 	ses.authstate.authdone = 0;
 	ses.authstate.pw = NULL;
 	ses.authstate.username = NULL;
+	ses.authstate.printableuser = NULL;
 	ses.authstate.authtypes = 0;
 #ifdef DROPBEAR_PUBKEY_AUTH
 	ses.authstate.authtypes |= AUTH_TYPE_PUBKEY;
@@ -62,6 +66,8 @@ static void authclear() {
 
 }
 
+/* Send a banner message if specified to the client. The client might
+ * ignore this, but possibly serves as a legal "no trespassing" sign */
 static void send_msg_userauth_banner() {
 
 	TRACE(("enter send_msg_userauth_banner"));
@@ -74,15 +80,18 @@ static void send_msg_userauth_banner() {
 
 	buf_putbyte(ses.writepayload, SSH_MSG_USERAUTH_BANNER);
 	buf_putstring(ses.writepayload, buf_getptr(ses.opts->banner,
-				ses.opts->banner->len), ses.opts->banner->size);
+				ses.opts->banner->len), ses.opts->banner->len);
 	buf_putstring(ses.writepayload, "en", 2);
 
 	encrypt_packet();
 	buf_free(ses.opts->banner);
+	ses.opts->banner = NULL;
 
 	TRACE(("leave send_msg_userauth_banner"));
 }
 
+/* handle a userauth request, check validity, pass to password or pubkey
+ * checking, and handle success or failure */
 void recv_msg_userauth_request() {
 
 	unsigned char *username, *servicename, *methodname;
@@ -117,76 +126,113 @@ void recv_msg_userauth_request() {
 		dropbear_exit("unknown service in auth");
 	}
 
+	/* user wants to know what methods are supported */
 	if (methodlen == AUTH_METHOD_NONE_LEN &&
 			strncmp(methodname, AUTH_METHOD_NONE,
 				AUTH_METHOD_NONE_LEN) == 0) {
 		send_msg_userauth_failure(0, 0);
-	} else if (checkusername(username, userlen) == CHECK_USER_RETURN) {
+		goto out;
+	}
+	
+	/* check username is good before continuing */
+	if (checkusername(username, userlen) == DROPBEAR_FAILURE) {
 		/* username is invalid/no shell/etc - send failure */
 		TRACE(("sending checkusername failure"));
 		send_msg_userauth_failure(0, 1);
+		goto out;
+	}
+
 #ifdef DROPBEAR_PASSWORD_AUTH
-	} else if (methodlen == AUTH_METHOD_PASSWORD_LEN &&
+	/* user wants to try password auth */
+	if (methodlen == AUTH_METHOD_PASSWORD_LEN &&
 			strncmp(methodname, AUTH_METHOD_PASSWORD,
 				AUTH_METHOD_PASSWORD_LEN) == 0) {
 		passwordauth(username, userlen);
-#endif /* DROPBEAR_PASSWORD_AUTH */
+		goto out;
+	}
+#endif
+
 #ifdef DROPBEAR_PUBKEY_AUTH
-	} else if (methodlen == AUTH_METHOD_PUBKEY_LEN &&
+	/* user wants to try pubkey auth */
+	if (methodlen == AUTH_METHOD_PUBKEY_LEN &&
 			strncmp(methodname, AUTH_METHOD_PUBKEY,
 				AUTH_METHOD_PUBKEY_LEN) == 0) {
 		pubkeyauth(username, userlen);
-#endif /* DROPBEAR_PUBKEY_AUTH */
-	} else {
-		send_msg_userauth_failure(0, 1);
+		goto out;
 	}
+#endif
+
+	/* nothing matched, we just fail */
+	send_msg_userauth_failure(0, 1);
+
+out:
 
 	m_free(username);
 	m_free(servicename);
 	m_free(methodname);
 }
 
+/* Check that the username exists, has a non-empty password, and has a valid
+ * shell.
+ * returns DROPBEAR_SUCCESS on valid username, DROPBEAR_FAILURE on failure */
 static int checkusername(unsigned char *username, unsigned int userlen) {
 
 	char* shell;
+	char* newprintableuser;
 	
 	TRACE(("enter checkusername"));
 	if (userlen > MAX_USERNAME_LEN) {
-		return CHECK_USER_RETURN;
+		return DROPBEAR_FAILURE;
 	}
 
+	newprintableuser = stripcontrol(username);
+
 	/* new user or username has changed */
 	if (ses.authstate.username == NULL ||
 		strcmp(username, ses.authstate.username) != 0) {
 			/* the username needs resetting */
 			if (ses.authstate.username != NULL) {
-				dropbear_log(LOG_AUTHPRIV | LOG_WARNING,
-						"client trying multiple usernames: %s and %s",
-						ses.authstate.username, username);
+				dropbear_log(LOG_WARNING,
+					"client trying multiple usernames: '%s' and '%s' from %s",
+					ses.authstate.printableuser, newprintableuser,
+					ses.addrstring);
 				m_free(ses.authstate.username);
+				m_free(ses.authstate.printableuser);
 			}
 			authclear();
 			ses.authstate.pw = getpwnam((char*)username);
 			ses.authstate.username = strdup(username);
+			ses.authstate.printableuser = newprintableuser;
 	}
 
 	/* check that user exists */
 	if (ses.authstate.pw == NULL) {
 		TRACE(("leave checkusername: user doesn't exist"));
+		dropbear_log(LOG_WARNING,
+				"login attempt for nonexistant user '%s' from %s",
+				username, ses.addrstring);
 		send_msg_userauth_failure(0, 1);
-		return CHECK_USER_RETURN;
+		return DROPBEAR_FAILURE;
+	}
+
+	/* check for non-root if desired */
+	if (ses.opts->norootlogin && ses.authstate.pw->pw_uid == 0) {
+		TRACE(("leave checkusername: root login disabled"));
+		dropbear_log(LOG_WARNING, "root login denied");
+		send_msg_userauth_failure(0, 1);
+		return DROPBEAR_FAILURE;
 	}
 
 	/* check for an empty password */
 	if (ses.authstate.pw->pw_passwd[0] == '\0') {
 		TRACE(("leave checkusername: empty pword"));
+		dropbear_log(LOG_WARNING, "empty password login denied");
 		send_msg_userauth_failure(0, 1);
-		return CHECK_USER_RETURN;
+		return DROPBEAR_FAILURE;
 	}
 
 	TRACE(("shell is %s", ses.authstate.pw->pw_shell));
 	/* check that the shell is valid */
-	/* XXX - todo check this is correct: empty shell is ok */
 	if (ses.authstate.pw->pw_shell[0] == '\0') {
 		goto goodshell;
 	}
@@ -201,22 +247,25 @@ static int checkusername(unsigned char *username, unsigned int userlen) {
 	/* no matching shell */
 	endusershell();
 	TRACE(("no matching shell"));
+	dropbear_log(LOG_WARNING, "invalid shell login denied");
 	send_msg_userauth_failure(0, 1);
-	return CHECK_USER_RETURN;
+	return DROPBEAR_FAILURE;
 	
 goodshell:
 	endusershell();
 	TRACE(("matching shell"));
 
-	TRACE(("uid = %d\n", ses.authstate.pw->pw_uid));
+	TRACE(("uid = %d", ses.authstate.pw->pw_uid));
 	TRACE(("leave checkusername"));
-	return CHECK_USER_CONTINUE;
+	return DROPBEAR_SUCCESS;
 
 }
 
-/* partial indicates whether to set the "partial success" flag,
+/* Send a failure message to the client, in responds to a userauth_request.
+ * Partial indicates whether to set the "partial success" flag,
  * incrfail is whether to count this failure in the failure count (which
- * is limited */
+ * is limited. This function also handles disconnection after too many
+ * failures */
 void send_msg_userauth_failure(int partial, int incrfail) {
 
 	buffer *typebuf;
@@ -262,27 +311,28 @@ void send_msg_userauth_failure(int partial, int incrfail) {
 		if (ses.authstate.username == NULL) {
 			userstr = "is unknown!!!!";
 		} else {
-			userstr = ses.authstate.username;
+			userstr = ses.authstate.printableuser;
 		}
-		userstr=  "blargh";
 		dropbear_exit("Max auth tries reached - user %s", userstr);
 	}
 	
 	TRACE(("leave send_msg_userauth_failure"));
 }
 
-/* only to be called when writebuf is empty */
+/* Send a success message to the user, and set the "authdone" flag */
 void send_msg_userauth_success() {
 
 	TRACE(("enter send_msg_userauth_success"));
 
 	CHECKCLEARTOWRITE();
 
+	assert(ses.authstate.username);
+
 	buf_putbyte(ses.writepayload, SSH_MSG_USERAUTH_SUCCESS);
 	encrypt_packet();
 
 	ses.authstate.authdone = 1;
-	close(ses.childpipe);
+	close(ses.childpipe); /* remove from the list of pre-auth sockets */
 	TRACE(("leave send_msg_userauth_success"));
 
 }
