@@ -45,8 +45,7 @@
 
 #define RECV_MAXWINDOW 6000 /* tweak */
 #define RECV_MAXPACKET 1400 /* tweak */
-#define RECV_MINWINDOW 10000 /* when we get below this, we send a windowadjust,
-								not currently used */
+#define RECV_MINWINDOW 19000 /* when we get below this, we send a windowadjust */
 
 /* a simpler way to define that we need code for listeners */
 #if !defined(DISABLE_X11FWD) || !defined(DISABLE_AUTHFWD) || \
