@@ -111,16 +111,24 @@ static void _dropbear_exit(int exitcode, const char* format, va_list param) {
 
 	if (!sessinitdone) {
 		/* before session init */
-		snprintf(fmtbuf, sizeof(fmtbuf), "exited: %s", format);
+		snprintf(fmtbuf, sizeof(fmtbuf), 
+				"premature exit: %s", format);
 	} else if (ses.authstate.authdone) {
 		/* user has authenticated */
 		snprintf(fmtbuf, sizeof(fmtbuf),
-				"exited after userauth (%s) from %s: %s", 
+				"exit after auth (%s) from %s: %s", 
 				ses.authstate.printableuser, ses.addrstring, format);
+	} else if (ses.authstate.username) {
+		/* we have a potential user */
+		snprintf(fmtbuf, sizeof(fmtbuf), 
+				"exit before auth (user '%s', %d fails) from %s: %s",
+				ses.authstate.username, ses.authstate.failcount,
+				ses.addrstring, format);
 	} else {
 		/* before userauth */
-		snprintf(fmtbuf, sizeof(fmtbuf), "exited before userauth: %s",
-				format);
+		snprintf(fmtbuf, sizeof(fmtbuf), 
+				"exit before auth from %s: %s",
+				ses.addrstring, format);
 	}
 
 	_dropbear_log(LOG_INFO, fmtbuf, param);
@@ -328,31 +336,6 @@ char * stripcontrol(const char * text) {
 }
 			
 
-
-
-/* returns the current position on success, or -1 on failure */
-int readln(int fd, char* buf, int count) {
-	
-	char in;
-	int pos = 0;
-	
-	/* leave space to null-terminate */
-	while (pos < count-1) {
-		if (atomicio(read, fd, &in, 1) < 0) {
-			break;
-		}
-
-		if (in == '\n' || in == '\r') {
-			break;
-		}
-
-		buf[pos] = in;
-		pos++;
-	}
-	buf[pos] = '\0';
-	return pos+1;
-}
-
 /* reads the contents of filename into the buffer buf, from the current
  * position, either to the end of the file, or the buffer being full.
  * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
@@ -364,7 +347,7 @@ int buf_readfile(buffer* buf, const char* filename) {
 
 	fd = open(filename, O_RDONLY);
 
-	if (fd == -1) {
+	if (fd < 0) {
 		close(fd);
 		return DROPBEAR_FAILURE;
 	}
@@ -374,7 +357,7 @@ int buf_readfile(buffer* buf, const char* filename) {
 		len = read(fd, buf_getwriteptr(buf, maxlen),
 				maxlen);
 		buf_incrwritepos(buf, len);
-	} while (len != maxlen && len > 0);
+	} while (len < maxlen && len > 0);
 
 	close(fd);
 	return DROPBEAR_SUCCESS;
@@ -415,7 +398,6 @@ void * m_malloc(size_t size) {
 void __m_free(void* ptr) {
 	if (ptr != NULL) {
 		free(ptr);
-		ptr = NULL;
 	}
 }
 
