@@ -36,7 +36,12 @@ static void sigintterm_handler(int fish);
 
 static int childpipes[MAX_UNAUTH_CLIENTS];
 
+#if defined(DBMULTI_DROPBEAR) || !defined(DROPBEAR_MULTI)
+#if defined(DBMULTI_DROPBEAR) && defined(DROPBEAR_MULTI)
+int dropbear_main(int argc, char ** argv) {
+#else
 int main(int argc, char ** argv) {
+#endif
 	
 	fd_set fds;
 	struct timeval seltimeout;
@@ -132,9 +137,10 @@ int main(int argc, char ** argv) {
 
 		/* pre-authentication clients */
 		for (i = 0; i < MAX_UNAUTH_CLIENTS; i++) {
-			if (childpipes[i] >= 0) 
-			FD_SET(childpipes[i], &fds);
-			maxsock = MAX(maxsock, childpipes[i]);
+			if (childpipes[i] >= 0) {
+				FD_SET(childpipes[i], &fds);
+				maxsock = MAX(maxsock, childpipes[i]);
+			}
 		}
 
 		val = select(maxsock+1, &fds, NULL, NULL, &seltimeout);
@@ -239,6 +245,7 @@ int main(int argc, char ** argv) {
 	/* don't reach here */
 	return -1;
 }
+#endif
 
 /* catch + reap zombie children */
 static void sigchld_handler(int fish) {
