@@ -25,7 +25,7 @@
 #include "includes.h"
 #include "packet.h"
 #include "session.h"
-#include "util.h"
+#include "dbutil.h"
 #include "ssh.h"
 #include "algo.h"
 #include "buffer.h"
@@ -38,6 +38,8 @@
 static void read_packet_init();
 static void process_postauth_packet(unsigned int type);
 static void recv_unimplemented();
+static void writemac(buffer * outputbuffer, buffer * clearwritebuf);
+static int checkmac(buffer* hashbuf, buffer* readbuf);
 
 #define ZLIB_COMPRESS_INCR 20 /* this is 12 bytes + 0.1% of 8000 bytes */
 #define ZLIB_DECOMPRESS_INCR 100
@@ -55,10 +57,12 @@ void write_packet() {
 	TRACE(("enter write_packet"));
 	assert(!isempty(&ses.writequeue));
 
+	/* Get the next buffer in the queue of encrypted packets to write*/
 	writebuf = (buffer*)examine(&ses.writequeue);
 
 	len = writebuf->len - writebuf->pos;
 	assert(len > 0);
+	/* Try to write as much as possible */
 	written = write(ses.sock, buf_getptr(writebuf, len), len);
 
 	if (written < 0) {
@@ -71,20 +75,22 @@ void write_packet() {
 	} 
 
 	if (written == 0) {
-		dropbear_close("remote host closed connection");
+		session_remoteclosed();
 	}
 
 	if (written == len) {
+		/* We've finished with the packet, free it */
 		dequeue(&ses.writequeue);
 		buf_free(writebuf);
 	} else {
+		/* More packet left to write, leave it in the queue for later */
 		buf_incrpos(writebuf, written);
 	}
 
 	TRACE(("leave write_packet"));
 }
 
-/* non-blocking function reading available portion of a packet into the
+/* Non-blocking function reading available portion of a packet into the
  * ses's buffer, decrypting the length if encrypted, decrypting the
  * full portion if possible */
 void read_packet() {
@@ -97,41 +103,50 @@ void read_packet() {
 	blocksize = ses.keys->recv_algo_crypt->blocksize;
 	
 	if (ses.readbuf == NULL || ses.readbuf->len < blocksize) {
-		/* don't know the packetsize since we haven't got the first block to
-		 * decrypt */
+		/* In the first blocksize of a packet */
+
+		/* Read the first blocksize of the packet, so we can decrypt it and
+		 * find the length of the whole packet */
 		read_packet_init();
-		/* we should return so that we can select(), to make sure that the
-		 * next read won't return 0 simply because there are no more bytes */
-		TRACE(("leave read_packet: packetinit done"));
-		return;
+
+		/* If we don't have the length of decryptreadbuf, we didn't read
+		 * a whole blocksize and should exit */
+		if (ses.decryptreadbuf->len == 0) {
+			TRACE(("leave read_packet: packetinit done"));
+			return;
+		}
 	}
 
+	/* Attempt to read the remainder of the packet, note that there
+	 * mightn't be any available (EAGAIN) */
 	assert(ses.readbuf != NULL);
 	maxlen = ses.readbuf->len - ses.readbuf->pos;
 	len = read(ses.sock, buf_getptr(ses.readbuf, maxlen), maxlen);
 	buf_incrpos(ses.readbuf, len);
 
 	if (len == 0) {
-		dropbear_close("remote host closed connection");
+		session_remoteclosed();
 	}
 
 	if (len < 0) {
-		if (errno == EINTR) {
-			TRACE(("leave read_packet: EINTR"));
+		if (errno == EINTR || errno == EAGAIN) {
+			TRACE(("leave read_packet: EINTR or EAGAIN"));
 			return;
 		} else {
-			dropbear_exit("error reading");
+			dropbear_exit("error reading: %s", strerror(errno));
 		}
 	}
 
-	if (len == maxlen) {
+	if ((unsigned int)len == maxlen) {
+		/* The whole packet has been read */
 		decrypt_packet();
-		/* process_packet() will handle the packet from the main select loop */
+		/* The main select() loop in session.h will process_packet() to
+		 * handle the packet contents... */
 	}
 	TRACE(("leave read_packet"));
 }
 
-/* function used to read the initial portion of a packet, and determine the
+/* Function used to read the initial portion of a packet, and determine the
  * length. Only called during the first BLOCKSIZE of a packet. */
 static void read_packet_init() {
 
@@ -157,14 +172,19 @@ static void read_packet_init() {
 	len = read(ses.sock, buf_getwriteptr(ses.readbuf, maxlen),
 			maxlen);
 	if (len == 0) {
-		dropbear_close("remote host closed connection");
+		session_remoteclosed();
 	}
 	if (len < 0) {
-		dropbear_exit("error reading");
+		if (errno == EINTR) {
+			TRACE(("leave read_packet_init: EINTR"));
+			return;
+		}
+		dropbear_exit("error reading: %s", strerror(errno));
 	}
 
 	buf_incrwritepos(ses.readbuf, len);
-	if (len != maxlen) {
+
+	if ((unsigned int)len != maxlen) {
 		/* don't have enough bytes to determine length, get next time */
 		return;
 	}
@@ -172,19 +192,20 @@ static void read_packet_init() {
 	/* now we have the first block, need to get packet length, so we decrypt
 	 * the first block (only need first 4 bytes) */
 	buf_setpos(ses.readbuf, 0);
-	buf_setpos(ses.decryptreadbuf, 0);
-	buf_setlen(ses.decryptreadbuf, 0);
-	if (ses.keys->recv_algo_crypt->cipherdesc != NULL) {
+	if (ses.keys->recv_algo_crypt->cipherdesc == NULL) {
+		/* copy it */
+		memcpy(buf_getwriteptr(ses.decryptreadbuf, blocksize),
+				buf_getptr(ses.readbuf, blocksize),
+				blocksize);
+	} else {
+		/* decrypt it */
 		if (cbc_decrypt(buf_getptr(ses.readbuf, blocksize), 
 					buf_getwriteptr(ses.decryptreadbuf,blocksize),
 					&ses.keys->recv_symmetric_struct) != CRYPT_OK) {
 			dropbear_exit("error decrypting");
 		}
-	} else {
-		memcpy(buf_getwriteptr(ses.decryptreadbuf, blocksize), 
-				buf_getptr(ses.readbuf, blocksize), blocksize);
 	}
-	buf_incrlen(ses.decryptreadbuf, blocksize);
+	buf_setlen(ses.decryptreadbuf, blocksize);
 	len = buf_getint(ses.decryptreadbuf) + 4 + macsize;
 
 	buf_setpos(ses.readbuf, blocksize);
@@ -204,13 +225,10 @@ static void read_packet_init() {
 /* handle the received packet */
 void decrypt_packet() {
 
-	hmac_state hmac;
-	unsigned char seqbuf[4];
 	unsigned char blocksize;
 	unsigned char macsize;
 	unsigned int padlen;
 	unsigned int len;
-	unsigned char *recvmac;
 
 	TRACE(("enter decrypt_packet"));
 	blocksize = ses.keys->recv_algo_crypt->blocksize;
@@ -226,7 +244,13 @@ void decrypt_packet() {
 	buf_setpos(ses.decryptreadbuf, blocksize);
 
 	/* decrypt if encryption is set, memcpy otherwise */
-	if (ses.keys->recv_algo_crypt->cipherdesc != NULL) {
+	if (ses.keys->recv_algo_crypt->cipherdesc == NULL) {
+		/* copy it */
+		len = ses.readbuf->len - macsize - blocksize;
+		memcpy(buf_getwriteptr(ses.decryptreadbuf, len),
+				buf_getptr(ses.readbuf, len), len);
+	} else {
+		/* decrypt */
 		while (ses.readbuf->pos < ses.readbuf->len - macsize) {
 			if (cbc_decrypt(buf_getptr(ses.readbuf, blocksize), 
 						buf_getwriteptr(ses.decryptreadbuf, blocksize),
@@ -236,50 +260,13 @@ void decrypt_packet() {
 			buf_incrpos(ses.readbuf, blocksize);
 			buf_incrwritepos(ses.decryptreadbuf, blocksize);
 		}
-	} else {
-		/* no encryption */
-		len = ses.readbuf->len - ses.decryptreadbuf->pos;
-		memcpy(buf_getwriteptr(ses.decryptreadbuf, len),
-				buf_getptr(ses.readbuf, len), len);
 	}
 
-	if (macsize > 0) {
-		/* calculate the mac */
-		if (hmac_init(&hmac, 
-					find_hash(ses.keys->recv_algo_mac->hashdesc->name), 
-					ses.keys->recvmackey, 
-					ses.keys->recv_algo_mac->keysize) 
-					!= CRYPT_OK) {
-			dropbear_exit("HMAC error");
-		}
-	
-		/* sequence number */
-		STORE32H(ses.recvseq, seqbuf);
-		if (hmac_process(&hmac, seqbuf, 4) != CRYPT_OK) {
-			dropbear_exit("HMAC error");
-		}
-	
-		buf_setpos(ses.decryptreadbuf, 0);
-		len = ses.decryptreadbuf->len;
-		if (hmac_process(&hmac, buf_getptr(ses.decryptreadbuf, len), len)
-				!= CRYPT_OK) {
-			dropbear_exit("HMAC error");
-		}
-	
-		recvmac = (unsigned char*)m_malloc(macsize);
-		if (hmac_done(&hmac, recvmac) != CRYPT_OK) {
-			dropbear_exit("HMAC error");
-		}
-	
-		/* compare the hash */
-		buf_setpos(ses.readbuf, ses.readbuf->len - macsize);
-		if (memcmp(recvmac, buf_getptr(ses.readbuf, macsize),
-					macsize) != 0) {
-			dropbear_exit("Integrity error");
-		}
-		m_free(recvmac);
-
-	} /* hash */
+	/* check the hmac */
+	buf_setpos(ses.readbuf, ses.readbuf->len - macsize);
+	if (checkmac(ses.readbuf, ses.decryptreadbuf) != DROPBEAR_SUCCESS) {
+		dropbear_exit("Integrity error");
+	}
 
 	/* readbuf no longer required */
 	buf_free(ses.readbuf);
@@ -308,8 +295,7 @@ void decrypt_packet() {
 	{
 		/* copy payload */
 		ses.payload = buf_new(len);
-		memcpy(ses.payload->data, buf_getptr(ses.decryptreadbuf, len),
-				len);
+		memcpy(ses.payload->data, buf_getptr(ses.decryptreadbuf, len), len);
 		buf_incrlen(ses.payload, len);
 	}
 
@@ -322,6 +308,56 @@ void decrypt_packet() {
 	TRACE(("leave decrypt_packet"));
 }
 
+/* Checks the mac in hashbuf, for the data in readbuf.
+ * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
+static int checkmac(buffer* macbuf, buffer* sourcebuf) {
+
+	unsigned char macsize;
+	hmac_state hmac;
+	unsigned char tempbuf[MAX_MAC_LEN];
+	unsigned long hashsize;
+	int len;
+
+	macsize = ses.keys->recv_algo_mac->hashsize;
+
+	if (macsize == 0) {
+		return DROPBEAR_SUCCESS;
+	}
+
+	/* calculate the mac */
+	if (hmac_init(&hmac, 
+				find_hash(ses.keys->recv_algo_mac->hashdesc->name), 
+				ses.keys->recvmackey, 
+				ses.keys->recv_algo_mac->keysize) 
+				!= CRYPT_OK) {
+		dropbear_exit("HMAC error");
+	}
+	
+	/* sequence number */
+	STORE32H(ses.recvseq, tempbuf);
+	if (hmac_process(&hmac, tempbuf, 4) != CRYPT_OK) {
+		dropbear_exit("HMAC error");
+	}
+
+	buf_setpos(sourcebuf, 0);
+	len = sourcebuf->len;
+	if (hmac_process(&hmac, buf_getptr(sourcebuf, len), len) != CRYPT_OK) {
+		dropbear_exit("HMAC error");
+	}
+
+	hashsize = sizeof(tempbuf);
+	if (hmac_done(&hmac, tempbuf, &hashsize) != CRYPT_OK) {
+		dropbear_exit("HMAC error");
+	}
+
+	/* compare the hash */
+	if (memcmp(tempbuf, buf_getptr(macbuf, macsize), macsize) != 0) {
+		return DROPBEAR_FAILURE;
+	} else {
+		return DROPBEAR_SUCCESS;
+	}
+}
+
 #ifndef DISABLE_ZLIB
 /* returns a pointer to a newly created buffer */
 static buffer* buf_decompress(buffer* buf, unsigned int len) {
@@ -529,8 +565,6 @@ void encrypt_packet() {
 
 	unsigned char padlen;
 	unsigned char blocksize, macsize;
-	hmac_state hmac;
-	unsigned char seqbuf[4];
 	buffer * writebuf; /* the packet which will go on the wire */
 	buffer * clearwritebuf; /* unencrypted, possibly compressed */
 	
@@ -539,8 +573,8 @@ void encrypt_packet() {
 	blocksize = ses.keys->trans_algo_crypt->blocksize;
 	macsize = ses.keys->trans_algo_mac->hashsize;
 
-	/* packet encrypted len is payload+5, then worst case is if we are 3 away
-	 * from a blocksize multiple, in which case we need to pad to the
+	/* Encrypted packet len is payload+5, then worst case is if we are 3 away
+	 * from a blocksize multiple. In which case we need to pad to the
 	 * multiple, then add another blocksize (or MIN_PACKET_LEN) */
 	clearwritebuf = buf_new((ses.writepayload->len+4+1) + MIN_PACKET_LEN + 3
 #ifndef DISABLE_ZLIB
@@ -555,8 +589,7 @@ void encrypt_packet() {
 #ifndef DISABLE_ZLIB
 	/* compression */
 	if (ses.keys->trans_algo_comp == DROPBEAR_COMP_ZLIB) {
-		buf_compress(clearwritebuf, 
-					ses.writepayload, ses.writepayload->len);
+		buf_compress(clearwritebuf, ses.writepayload, ses.writepayload->len);
 	} else
 #endif
 	{
@@ -570,7 +603,8 @@ void encrypt_packet() {
 	buf_setpos(ses.writepayload, 0);
 	buf_setlen(ses.writepayload, 0);
 
-	/* length of padding */
+	/* length of padding - packet length must be a multiple of blocksize,
+	 * with a minimum of 4 bytes of padding */
 	padlen = blocksize - (clearwritebuf->len) % blocksize;
 	if (padlen < 4) {
 		padlen += blocksize;
@@ -581,7 +615,7 @@ void encrypt_packet() {
 	}
 
 	buf_setpos(clearwritebuf, 0);
-	/* packet length excl the packetlen uint32 */
+	/* packet length excluding the packetlength uint32 */
 	buf_putint(clearwritebuf, clearwritebuf->len + padlen - 4);
 
 	/* padding len */
@@ -597,7 +631,13 @@ void encrypt_packet() {
 	 * wire by writepacket() */
 	writebuf = buf_new(clearwritebuf->len + macsize);
 
-	if (ses.keys->trans_algo_crypt->cipherdesc != NULL) {
+	if (ses.keys->trans_algo_crypt->cipherdesc == NULL) {
+		/* copy it */
+		memcpy(buf_getwriteptr(writebuf, clearwritebuf->len),
+				buf_getptr(clearwritebuf, clearwritebuf->len),
+				clearwritebuf->len);
+		buf_incrwritepos(writebuf, clearwritebuf->len);
+	} else {
 		/* encrypt it */
 		while (clearwritebuf->pos < clearwritebuf->len) {
 			if (cbc_encrypt(buf_getptr(clearwritebuf, blocksize),
@@ -608,16 +648,38 @@ void encrypt_packet() {
 			buf_incrpos(clearwritebuf, blocksize);
 			buf_incrwritepos(writebuf, blocksize);
 		}
-	} else {
-		/* no encryption */
-		memcpy(buf_getwriteptr(writebuf, clearwritebuf->len),
-			buf_getptr(clearwritebuf, clearwritebuf->len),
-					clearwritebuf->len);
-		buf_incrwritepos(writebuf, clearwritebuf->len);
 	}
 
-
 	/* now add a hmac and we're done */
+	writemac(writebuf, clearwritebuf);
+
+	/* clearwritebuf is finished with */
+	buf_free(clearwritebuf);
+
+	/* enqueue the packet for sending */
+	buf_setpos(writebuf, 0);
+	enqueue(&ses.writequeue, (void*)writebuf);
+
+	/* Update counts */
+	ses.kexstate.datatrans += writebuf->len;
+	ses.transseq++;
+
+	TRACE(("leave encrypt_packet()"));
+}
+
+
+/* Create the packet mac, and append H(seqno|clearbuf) to the output */
+static void writemac(buffer * outputbuffer, buffer * clearwritebuf) {
+
+	int macsize;
+	unsigned char seqbuf[4];
+	unsigned long hashsize;
+	hmac_state hmac;
+
+	TRACE(("enter writemac"));
+
+	macsize = ses.keys->trans_algo_mac->hashsize;
+
 	if (macsize > 0) {
 		/* calculate the mac */
 		if (hmac_init(&hmac, 
@@ -642,25 +704,14 @@ void encrypt_packet() {
 			dropbear_exit("HMAC error");
 		}
 	
-		if (hmac_done(&hmac, buf_getwriteptr(writebuf, macsize)) 
+		hashsize = macsize;
+		if (hmac_done(&hmac, buf_getwriteptr(outputbuffer, macsize), &hashsize) 
 				!= CRYPT_OK) {
 			dropbear_exit("HMAC error");
 		}
-		buf_incrlen(writebuf, macsize);
-	
-	} /* hash */
-
-	/* clearwritebuf is finished with */
-	buf_free(clearwritebuf);
-
-	ses.transseq++;
-
-	buf_setpos(writebuf, 0);
-	enqueue(&ses.writequeue, (void*)writebuf);
-
-	ses.kexstate.datatrans += writebuf->len;
-
-	TRACE(("leave encrypt_packet()"));
+		buf_incrwritepos(outputbuffer, macsize);
+	}
+	TRACE(("leave writemac"));
 }
 
 #ifndef DISABLE_ZLIB
