@@ -22,11 +22,8 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE. */
 
-#include <stdlib.h>
-#include <string.h>
-
-#include "options.h"
-#include "util.h"
+#include "includes.h"
+#include "dbutil.h"
 #include "signkey.h"
 #include "buffer.h"
 #include "ssh.h"
@@ -82,7 +79,7 @@ int buf_get_pub_key(buffer *buf, sign_key *key, int type) {
 	
 }
 
-/* returns 0 on success, -1 on fail */
+/* returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
 int buf_get_priv_key(buffer *buf, sign_key *key, int type) {
 
 	unsigned char* ident;
@@ -100,6 +97,9 @@ int buf_get_priv_key(buffer *buf, sign_key *key, int type) {
 		dss_key_free(key->dsskey);
 		key->dsskey = (dss_key*)m_malloc(sizeof(dss_key));
 		ret = buf_get_dss_priv_key(buf, key->dsskey);
+		if (ret == DROPBEAR_FAILURE) {
+			m_free(key->dsskey);
+		}
 		TRACE(("leave buf_get_priv_key: done get dss"));
 		return ret;
 	}
@@ -112,6 +112,9 @@ int buf_get_priv_key(buffer *buf, sign_key *key, int type) {
 		rsa_key_free(key->rsakey);
 		key->rsakey = (rsa_key*)m_malloc(sizeof(rsa_key));
 		ret = buf_get_rsa_priv_key(buf, key->rsakey);
+		if (ret == DROPBEAR_FAILURE) {
+			m_free(key->rsakey);
+		}
 		TRACE(("leave buf_get_priv_key: done get rsa"));
 		return ret;
 	}
@@ -120,7 +123,7 @@ int buf_get_priv_key(buffer *buf, sign_key *key, int type) {
 	m_free(ident);
 	
 	TRACE(("leave buf_get_priv_key"));
-	return -1;
+	return DROPBEAR_FAILURE;
 	
 }
 
@@ -183,9 +186,11 @@ void sign_key_free(sign_key *key) {
 
 #ifdef DROPBEAR_DSS
 	dss_key_free(key->dsskey);
+	key->dsskey = NULL;
 #endif
 #ifdef DROPBEAR_RSA
 	rsa_key_free(key->rsakey);
+	key->rsakey = NULL;
 #endif
 
 	m_free(key);
